class Admin::EpisodesController < AdministrationController

  before_filter :get_show

  def show
    @episode = Episode.find(params[:id])
  end
  
  def new
    @episode = Episode.new
    # seta o número do episódio para 1 a mais 
    # do último episódio cadastrado para esta temporada
    @episode.number = (Episode.scoped_by_show_id_and_season_number(@show.id, params[:season_number] || 1).maximum(:number) || 0) + 1
  end
  
  def create
    authorize! :create, Episode
    @episode = Episode.new(params[:episode])
    @episode.show = @show
    if @episode.save
      flash[:notice] = "Episódio criado com sucesso."
      redirect_to [ :admin, @show, @episode ]
    else
      render :action => 'new'
    end
  end
  
  def edit
    @episode = Episode.find(params[:id])
  end
  
	def update
    authorize! :update, Episode
    @episode = Episode.find(params[:id])
    if @episode.update_attributes(params[:episode])
      flash[:notice] = "Episódio editado com sucesso."
      redirect_to [ :admin, @show, @episode ]
    else
      render :action => 'edit'
    end
  end
  
  def destroy
    authorize! :destroy, Episode
    @episode = Episode.find(params[:id])
    @episode.destroy
    flash[:notice] = "Episódio excluido com sucesso."
    redirect_to admin_show_episodes_path(@show)
  end

  private

  def get_show
    @show = Show.find params[:show_id] 
  end

end
