class ApplicationController < ActionController::Base

  # garante acesso as views para estes métodos
  helper_method :current_user_session, :current_user

  protected

  # método usado para requerir login em um
  # controller (geralmente usado no before_filter)
  def require_user
    unless current_user
      flash[:notice] = "É preciso estar logado para acessar esta página"
      redirect_to root_path
    end
  end

  # quando qualquer uma dessas exceções forem geradas
  # executa este código
  # estas exceções são geradas por falta de permissão
  # tanto pelo CanCan quanto pelo Acl9
  rescue_from Acl9::AccessDenied, CanCan::AccessDenied do |exception|
    flash[:error] = "Permissão negada"
    redirect_to current_user ? user_path(current_user) : root_url
  end


  # método usado para requerir que o usuário não esteja logado
  # em um controller (geralmente usado no before_filter)
  def require_no_user
    if current_user
      flash[:notice] = "É preciso estar deslogado para acessar esta página"
      redirect_to user_path(current_user)
    end    
  end

  # usado no before_filter para setar a variavel
  # @user com o usuario correto
  def set_user
    if params[:user_id]
      @user = User.find params[:user_id]
    else
      @user = current_user
    end
  end

  private

  # retorna a sessão atual
  def current_user_session
    return @current_user_session if defined?(@current_user_session)
    @current_user_session = UserSession.find
  end

  # retorna o usuário logado
  def current_user
    return @current_user if defined?(@current_user)
    @current_user = current_user_session && current_user_session.user
  end

end
