module ApplicationHelper

  include ActionView::Helpers::FormTagHelper
  include WillPaginate::ViewHelpers::ActionView

  def errors_for(object)
    if object.errors.any?
      markaby do
        div.errors do 
          p "Erros encontrados:"
          ul do
            object.errors.full_messages.each {|msg| li msg}
          end
        end
      end.to_s.html_safe
    end
  end

  def markaby(&block)
    Markaby::Builder.new({}, self, &block)
  end

  def title(str)
    content_for :title do
      str
    end
  end

  def stars_for(object)
    render :partial => 'shared/stars', :locals => { :object => object }    
  end

  def favorites_for(object, user)
    render :partial => 'shared/favorites', :locals => { :object => object, :user => user }
  end

  def new_favorite(object, category, user = current_user)
    render :partial => 'shared/new_favorite', :locals => { :object => object, :category => category, :user => user }
  end

  def new_comment_for(object)
    render :partial => 'comments/new', :locals => { :item => object }
  end

  def comments_for(object, page = 1)
    render(object.comments.paginate :page => page, :per_page => Comment.per_page)
  end

  def friends_of(user)
    render(:partial => 'users/friends', :locals => { :user => user })
  end

  def submit_tag_with_class(value = "Save changes", options = {})
    options[:class] = "submit"
    submit_tag_without_class(value, options)
  end
  alias_method_chain :submit_tag, :class

  def will_paginate_with_tcc(collection = nil, options = {})
    options[:previous_label] = "Anterior"
    options[:next_label] = "Próximo"
    will_paginate_without_tcc(collection, options)
  end
  alias_method_chain :will_paginate, :tcc

  def role_select_tag user, field = 'roles'
    roles = ['admin', 'moderator']

    sel =  "<select id='#{field}' name='#{field}[]' multiple='multiple' size='5'>"
    roles.each do |role|
      selected = "selected='selected'" if user.has_role? role
      sel += "<option value='#{role}' #{selected}>#{role}</option>"
      selected = ''
    end
    sel += '</select>'
    sel.html_safe
  end

end
