class Favorite < ActiveRecord::Base

  scope :of_type, lambda { |type| where(:favorite_type => type)  }
  scope :from, lambda { |id| where(:user_id => id) }

  TYPES = [ "Estou Vendo", "Quero Ver", "Desisti" ]

  belongs_to :user
  belongs_to :favoritable, :polymorphic => true
  validates_uniqueness_of :favoritable_id, :scope => [:user_id, :favoritable_type]
  validates_presence_of :user_id, :favoritable_id, :favoritable_type, :favorite_type

  after_create :increase_counters
  after_update :update_counters
  after_destroy :decrease_counters

  private

  def increase_counters
    if self.favorite_type == 0
       favor = (self.favoritable.favorites_count || 0)
       favor += 1
       self.favoritable.favorites_count = favor
    end
    self.favoritable.save
  end

  def decrease_counters
    if self.favorite_type == 0
      favor = (self.favoritable.favorites_count || 0)
      favor -=1
      self.favoritable.favorites_count = favor
    end
    self.favoritable.save
  end

  def update_counters
    if self.favorite_type_changed?
      if self.favorite_type_was == 0
        self.favoritable.favorites_count -= 1
      end
      if self.favorite_type == 0
        self.favoritable.favorites_count += 1
      end
      self.favoritable.save
    end
  end
end
