class Forum < ActiveRecord::Base
  validates_presence_of :forum_category_id,
                        :name

  belongs_to :forum_category
  belongs_to :last_topic, :class_name => "Post"
  has_many  :topics, 
            :class_name => "Post",
            :conditions => ["posts.topic_id IS NULL"],
            :dependent => :destroy
  has_many :posts

  before_validation :set_position

  def set_position
    return unless self.position.nil?
    self.position = (Forum.scoped_by_forum_category_id(self.forum_category_id).maximum(:position) || 0) + 1
  end

  def increase_posts!
    self.increment!(:posts_count)
  end

  def decrease_posts!
    self.decrement!(:posts_count)
  end

  def increase_topics!
    self.increment!(:topics_count)
  end

  def decrease_topics!
    self.decrement!(:topics_count)
  end
    
  def update_last_topic(topic)
    self.update_attribute(:last_topic_id, (topic ? topic.id : nil))
  end
end
