class Post < ActiveRecord::Base

  validates_presence_of :user_id,
                        :forum_id,
                        :body

  belongs_to :forum, :counter_cache => :posts_count
  belongs_to :topic, :foreign_key => :topic_id, :class_name => "::Post", :counter_cache => :posts_count
  belongs_to :user
  belongs_to :last_post, :class_name => "::Post"
  has_many   :posts, :foreign_key => :topic_id
  after_save :increase_counters
  after_destroy :decrease_counters

  def topic?
    self.topic_id.nil?
  end

  def topic=(val)
    self.topic_id = val.id
    self.forum_id = val.forum_id
  end

  def increase_counters
    if topic?
      self.forum.increase_topics!
      self.forum.update_last_topic(self)
    else
      self.topic.update_last_post(self)
    end
  end

  def decrease_counters
    if topic?
      self.forum.decrease_topics!
      self.forum.update_last_topic(self.forum.topics.last)
    else
      self.topic.update_last_post(self.topic.posts.last)
    end
  end

  def update_last_post(post)
    pid = post.nil? ? nil : post.id
    self.update_attribute(:last_post_id, pid)
  end

  def last_post
    if self.topic? && (self.posts_count || 0) < 1
      self
    else
      Post.find(self.last_post_id)
    end
  end
end
