class Show < ActiveRecord::Base
  acts_as_rated
  is_favoritable
  is_searchable :name

  has_many :users,
           :through => :favorites

  has_many :episodes
  has_many :images, :as => :item
  has_many :comments, :as => :item, :order => 'created_at DESC'
  has_attached_file :image,
    :path => ":rails_root/public/system/shows/:id/:style/show_image.:extension",
    :url => "/system/shows/:id/:style/show_image.:extension",
    :default_url => '/images/defaults/show_:style.jpg',
    :styles => { :thumb => "80x80#", :medium => "150x150#" },
    :default_style => :medium

  validates_presence_of :name, :description, :seasons_number

  before_save :set_video

  def other_users(user)
    self.favorites.of_type(0).order('RAND()').where('favorites.user_id != ?', user.id).limit(5).collect {|f| f.user}
  end

  def set_video
    if (self.video =~ /http:\/\//)
      self.video = self.video.scan(/v=([^&]+)/).first.first
    end
  end
end
