class User < ActiveRecord::Base

  acts_as_authorization_subject

  acts_as_authentic
  has_friends
  is_searchable :name, :email

  has_many :favorites, :dependent => :destroy

  has_many :bans
  has_many :posts, :dependent => :destroy

  has_many :ratings, :foreign_key => :rater_id
  has_many :episode_views
  has_many :watched_episodes, :through => :episode_views, :class_name => "Episode", :source => :episode 
  has_attached_file :avatar,
    :styles => { :medium => "150x150#", :thumb => "65x65#" },
    :default_url => '/images/defaults/avatar_:style.jpg',
    :default_style => :medium

  validates_presence_of :name, :email

  def admin?
    self.has_role? :admin
  end

  def is_banned?
    # checa os bans do usuario
    self.bans.each do |ban|
      # checa se o ban ainda não foi terminado
      if ban.expires_at > DateTime.now
        # se ainda está banido, marca como banido
        self.banned = true
        self.save
        return ban
      end
    end
    # marca como não banido
    self.banned = false
    self.save
    false
  end

  def users_with_same_taste
    shows = self.favorites.select(:favoritable_id).collect {|s| s.favoritable_id}
    self.class.where('users.id NOT IN (?)', [self.id, self.friends].flatten).joins(:favorites).where('favorites.favoritable_id IN (?)', shows).group('favorites.user_id').order('RAND()').limit(5).all
  end

  def roles
    Role.all.select do |role|
      self.has_role?(role)
    end
  end
end
