# HasFriends
module ActiveRecord #:nodoc:
  module Has #:nodoc:
    module Friends
      class FriendsError < RuntimeError; end

      def self.included(base) #:nodoc:
        base.extend(ClassMethods)  
      end

      module ClassMethods
        def has_friends(*args)
          return if friendable? 
          send :include, ActiveRecord::Has::Friends::FriendsMethods

          has_many :friendships, :foreign_key => :buddy_id
          has_many :friends, :through => :friendships
          scope :not, lambda { |id| where('friend_id != ?', id) }
          after_destroy :remove_friendships
        end

        def friendable?
          self.included_modules.include?(ActiveRecord::Has::Friends::FriendsMethods)
        end
      end

      module FriendsMethods

        def self.included(base) #:nodoc:
          base.extend ClassMethods
        end


        def add_friend(friend)
          raise Exceptions::NotFriendableException unless friend.class.respond_to?(:friendable?) && friend.class.friendable?
          self.friendships.create({:friend_id => friend.id, :buddy_id => self.id})
        end

        def remove_friend(friend)
          return unless self.friends_with?(friend)
          self.friendships.scoped_by_friend_id(friend.id).destroy_all
        end

        def friends_with?(friend)
          self.friends.include?(friend)
        end

        def remove_friendships
          Friendship.where('friend_id = ? OR buddy_id = ?', self.id, self.id).destroy_all
        end

        module ClassMethods

        end

        module Exceptions
          class NotFriendableException < Exception
            def initialize
              super("Argument passed is not friendable")
            end
          end
        end
      end
    end
  end
end

ActiveRecord::Base.send :include, ActiveRecord::Has::Friends
