# IsSearchable
module ActiveRecord #:nodoc:
  module Is #:nodoc:
    module Searchable
      class SearchableError < RuntimeError; end

      def self.included(base) #:nodoc:
        base.extend(ClassMethods)  
      end

      module ClassMethods
        def is_searchable(*args)
          return if self.included_modules.include?(ActiveRecord::Is::Searchable::SearchableMethods)
          send :include, ActiveRecord::Is::Searchable::SearchableMethods

          cattr_accessor :searchable_fields
          self.searchable_fields = args
        end
      end

      module SearchableMethods

        def self.included(base) #:nodoc:
          base.extend ClassMethods
        end

        module ClassMethods

          def search(word, options = {})
            build_search(word, self.scoped(options))
          end

          private

          def build_search(word, scope)
            t = self.arel_table
            predicates = []
            self.searchable_fields.each do |field|
              predicates << t[field].matches("%#{word}%")
            end
            scope.where(build_predicates(predicates))
          end

          def build_predicates(predicates)
            predicates.inject {|p, p1| p = p.or(p1)}
          end

        end
      end
    end
  end
end

ActiveRecord::Base.send :include, ActiveRecord::Is::Searchable
